#!/usr/bin/python

from components.ram import Ram

class Caretaker:
	
	def __init__(self,ramSize):
		self.objects=dict() # Create empty objects pool
		# Add registers to pool
		for reg in ["MAR","MDR", "PC", "MBR", "SP","LV","CPP","TOS","OPC","H"]:
			self.objects[reg]=0
		self.objects["RAM"]=Ram(self,ramSize)
	
	def __getitem__(self,key):
		if key=="MBRU": # If we ask for unsigned
			return(abs(self.objects["MBR"]))
		elif key== "MBR":
			if self.objects[key] < 0:
				return(self.objects[key])
			elif self.objects[key]>>7==1: # If it a negative number (2 complement)
				return(-(self.objects[key]&0x7F))
			else:
				return(self.objects[key])
		return(self.objects[key])

	def __setitem__(self,key,value):
		if key!="RAM":
			if (-(2**7-1))>value and (key=="MBR" or key=="MBRU"):
				raise RuntimeError("Value {} cannot fit in MBR register (-2^7 minimum value)".format(value))
			elif value>(2**32-1) and (key!="MBR" and key!="MBRU"):
				raise RuntimeError("Value {} cannot fit in MBR register (2^32 minimum value)".format(value))

#		if key!="RAM":
#			if (value > (2**32) or value <(2**32)/2)  and key!="MBR" and key!="MBRU": # Check value fit in word
#				print("Warning word overflow: value {} on register {}".format(value,key))
#				value=value%(2**32) # Force to fit in word
#			elif (value > (2**8) or value <126) and key=="MBR" or key=="MBRU": # Check value fit in byte
#				print("Warning byte overflow: value {} on register {}".format(value,key))
#				value=abs(value)%256 # Force to fit in byte
		self.objects[key]=value
		
	def items(self):
		return(self.objects.items())	
	
